#!/usr/bin/env python
# -*- coding:utf-8 -*-

# GUI Interface of the dsysmon
#
# Requirements:
#  - python-wxgtk2.8 python-matplotlib
#
#  If you use Debian GNU/Linux or Ubuntu, you can use following command
#  line.
#
#  $ sudo apt-get install python-wxgtk2.8 python-matplotlib
#
# Usage:
#
#  1. run "dsysmon" before running this program.
#  2. The "dsysmon" creates named pipes to inside of the directory
#     /tmp/dsysmonPID/.
#  3. run this program dsm-gui with -d option. A "-d" option is
#     specified named pipe directory. For example, You can run this
#     program following command line.
#     # This assumed that the pid of "dsysmon" was 1500.
#
#      $ sudo python /path/to/dsm-gui.py -d /tmp/dsysmon1500/
#
#     The "sudo" is required when "dsysmon" was run by root
#     authentification.
#
# written by HAMANO kiyoto, soum corporation.

"""Dsysmon interface -- GUI interface for dsysmon"""

import getopt, os, re, select, sys, threading, time, wx

import matplotlib

matplotlib.use ('WXAgg')
from matplotlib.backends.backend_wxagg import FigureCanvasWxAgg as FigCanvas
from matplotlib.backends.backend_wx import NavigationToolbar2Wx
from matplotlib.figure import Figure

import numpy as np
import pylab


DSM_EVT_FIFO_READ_ID = wx.NewId ()

DSM_EVT_ST_READ_ID = wx.NewId ()

DSM_VERSION_STRING = "1.1.0"

DSM_FKBD_CTL = 'fkbd-ctl'

DSM_FKBD_MSG = 'fkbd-msg'

DSM_RTKL_CTL = 'rtkl-ctl'

DSM_RTKL_MSG = 'rtkl-msg'

DSM_LMS_CTL = 'lms-ctl'

DSM_LMS_MSG = 'lms-msg'

DSM_GRPH_DEFAULT_LEFT = 'netlog'

DSM_GRPH_DEFAULT_RGHT = 'hddlog'

DSM_GTITLE_REGEX = r'^((netlog)|(hddlog))([0-9]*)$'

DSM_LBL_FKBD_WORK = 'FoxyKBD'

DSM_LBL_FKBD_NOTWORK = 'FoxyKBD (Disabled)'

DSM_LBL_RTKL_WORK = 'RootkitLibra'

DSM_LBL_RTKL_STOP = 'RootkitLibra (Stop)'

DSM_LBL_RTKL_NOTWORK = 'RootkitLibra (Disabled)'

DSM_LBL_LMS_WORK = 'Lightweight Monitoring Service'

DSM_LBL_LMS_STOP = 'Lightweight Monitoring Service (Stop)'

DSM_LBL_LMS_NOTWORK = 'Lightweight Monitoring Service (Disabled)'

DSM_ERR_NODEV = ("E: you must specify devine name by command line "
                 "like -d DEVICE_NAME.")

DSM_ERR_NOGRAPH = "E: %(opt)s: graph style %(arg)s is not valid."

DSM_ERR_NOTDIGIT = "E: %(opt)s: argument %(arg)s is not digit."

DSM_ERR_INVALID_GTITLE = "E: %(opt)s: specified graph %(arg)s is not valid."

DSM_ERR_NOFIFO = ("E: %(opt)s %(path)s : please specify fifo root directory.")

DSM_ERR_GTITLE_SAMENAME = ("E: cannot specify same graph name to left and "
                           "right.")

DSM_ERR_TIMEOUT = ("E: select: timeout")

DSM_MSG_VERRION = ("Version (%(version)s) (%(timestamp)s)")

DSM_MSG_USAGE = ("Usage: %(program_name)s [options] -d FIFO_BASE_DIR\n"
                 "Options:\n"
                 "	-c <cpu_num(int)>\n"
                 "		Specify the number of cpu cores.\n"
                 "	-d <directory(path)>\n"
                 "		Specify fifo nodes root directory of the "
                 "dsysmon.\n"
                 "	-g <style(string)>\n"
                 "		Specify graph style.\n"
                 "		This accepts \"bar\" or \"plot\".\n"
                 "		The default value is \"plot\".\n"
                 "	-i <interval(int)>\n"
                 "		Specify redraw interval of the graph.\n"
                 "	-l <graph_name>\n"
                 "		Specify name of the left graph.\n"
                 "	-r <graph_name>\n"
                 "		Specify name of the right graph.\n"
                 "	-s <scale(int)>\n"
                 "		Specify scale of the graph.\n"
                 "	-h, --help\n"
                 "		Show this message.\n"
                 "	-v, --version\n"
                 "		Show version string.")

DSM_FIFO_READS = (DSM_FKBD_MSG, DSM_RTKL_MSG, DSM_LMS_MSG)

DSM_FIFO_WRITES = (DSM_FKBD_CTL, DSM_RTKL_CTL, DSM_LMS_CTL)

DSM_FIFOS = DSM_FIFO_READS + DSM_FIFO_WRITES


dsm_fifo_paths = {}

dsm_grph_style = "plot"

dsm_redraw_itvl = 2

dsm_cpu_number = 1

dsm_grph_indication = 90

dsm_cur_grphs = []

dsm_grph_left = DSM_GRPH_DEFAULT_LEFT

dsm_grph_right = DSM_GRPH_DEFAULT_RGHT


class DsysFifoReadEvent (wx.PyEvent):
    """This is a class that recieve/send FIFO events.
    """

    def __init__ (self, type, data):
        wx.PyEvent.__init__ (self)
        self.SetEventType (DSM_EVT_FIFO_READ_ID)
        self.type = type
        self.data = data

class DsysStReadEvent (wx.PyEvent):
    """This is a class that recieve/send statusbar events.
    """

    def __init__ (self, clear):
        wx.PyEvent.__init__ (self)
        self.SetEventType (DSM_EVT_ST_READ_ID)
        self.clear = clear

class DsysGraphData (threading.Thread):
    """This is a class that generates graph datas.
    """

    loop  = True

    def __init__ (self, name, frame):
        """Initialize a thread for generating graph data.
        """

        threading.Thread.__init__ (self)
        self.name  = name
        self.frame = frame
        self.sum   = 0
        self.lock  = threading.Lock ()

    def run (self):
        """Polling Specified time.
        """

        while (self.loop):
            time.sleep (dsm_redraw_itvl)
            self.send (self.sum)
            self.clear ()

    def stop (self):
        """Stop thread.
        """

        self.lock.acquire ()
        self.loop = False
        self.lock.release ()

    def clear (self):
        self.lock.acquire ()
        self.sum = 0
        self.lock.release ()

    def push (self, time, data):
        """Add new data.
        """

        self.lock.acquire ()
        self.sum += int (data)
        self.lock.release ()

    def send (self, data):
        """Send graph data to update graph
        """
        try:
            self.lock.acquire ()
            wx.PostEvent (self.frame, DsysFifoReadEvent (self.name, data))
        except:
            # main frame is not found. finishing program.
            pass
        finally:
            self.lock.release ()


class DsysFifo (threading.Thread):
    """This is a class that provide access to FIFO.
    """

    loop     = True
    fragment = {}
    graphs   = {}

    def __init__ (self, frame):
        """Initiazlize a thread for reading FIFO.
        """

        threading.Thread.__init__ (self)
        self.frame = frame
        for node in DSM_FIFOS:
            dsm_fifo_paths[node] = self.set (node)

    def run (self):
        """Start thread.
        """

        self.poll ()

    def stop (self):
        """Stop thread.
        """

        self.loop = False

    def update_gui_status (self, name):
        """Update GUI button status and GUI frame messages.
        """

        # Update FKBD status
        if name in [DSM_FKBD_CTL, DSM_FKBD_MSG]:
            label = self.frame.fkbd_frm.GetLabel ()

            if label != DSM_LBL_FKBD_WORK:
                self.frame.fkbd_frm.SetLabel (DSM_LBL_FKBD_WORK)

            if name == DSM_FKBD_MSG:
                self.frame.menu_gclr.Enable (True)

            if name == DSM_FKBD_CTL:
                self.frame.fkbd_bstrt.Enable (True)
                self.frame.fkbd_bstop.Enable (False)
                self.frame.fkbd_bchng.Enable (True)
                self.frame.fkbd_titvl.Enable (True)

        # Update RTKL status
        if name == DSM_RTKL_CTL:
            self.frame.rtkl_frm.SetLabel (DSM_LBL_RTKL_STOP)
            self.frame.rtkl_text.Enable (True)
            self.frame.rtkl_bstrt.Enable (True)
            self.frame.rtkl_bstop.Enable (False)
            self.frame.rtkl_bclr.Enable (True)
            self.frame.menu_rclr.Enable (True)
            if self.frame.menu_aclr.IsEnabled () == False:
                self.frame.menu_aclr.Enable (True)

        # Update LMS status
        if name == DSM_LMS_CTL:
            self.frame.lms_frm.SetLabel (DSM_LBL_LMS_STOP)
            self.frame.lms_text.Enable (True)
            self.frame.lms_bstrt.Enable (True)
            self.frame.lms_bstop.Enable (False)
            self.frame.lms_bitvl.Enable (True)
            self.frame.lms_titvl.Enable (True)
            self.frame.menu_lclr.Enable (True)
            if self.frame.menu_aclr.IsEnabled () == False:
                self.frame.menu_aclr.Enable (True)

    def set (self, name):
        """Set FIFO node pathes.
        """

        if not dsm_fifo_paths["base"]:
            return None

        path = dsm_fifo_paths["base"] + "/" + name
        if os.path.exists (path):
            self.update_gui_status (name)
            return path
        else:
            return None

    def write (self, type, value):
        """Write any data to FIFO.
        """

        if dsm_fifo_paths[type]:
            try:
                fd = os.open (dsm_fifo_paths[type], os.O_WRONLY)
                os.write (fd, value)
            finally:
                os.close (fd)

    def poll (self):
        """Polling fifo device (read)
        """

        fds = []
        # open fifo file descriptores
        for node in DSM_FIFO_READS:
            if not dsm_fifo_paths[node]:
                continue
            fd = os.open (dsm_fifo_paths[node], os.O_RDONLY | os.O_NONBLOCK)
            self.fragment[fd] = u''
            fds.append ([fd, node])
            if node == DSM_FKBD_MSG:
                for graph in dsm_cur_grphs:
                    self.graphs[graph] = DsysGraphData (graph, self.frame)
                    self.graphs[graph].start ()

        # select fifo file descriptores
        while self.loop:
            rfds, wfds, xfds = select.select (map (lambda x: x[0], fds), [], [])
            if not rfds:
                print >> sys.stderr, DSM_ERR_TIMEOUT
                continue
            for fd in rfds:
                for elem in fds:
                    if elem[0] == fd:
                        recieved = os.read(fd, 512).split('\n')
                        recieved, count = self.reassemble (elem[0], recieved)
                        for i in range (count):
                            # print "== " + elem[1] + ": " + recieved[i]
                            self.send (elem[1], recieved[i])

        # close file descriptores
        for fd in fds:
            os.close (fd[0])
        # finish thread for managing graph data.
        for graph in self.graphs:
            self.graphs[graph].stop()

    def reassemble (self, elem, recieved):
        """Reassemble recieved data to end with linefeed.
        """

        if self.fragment[elem] != u'':
            recieved[0] = self.fragment[elem] + recieved[0]
            self.fragment[elem] = u''

        count = len (recieved) -1
        if recieved[count] != '':
            self.fragment[elem] = (self.fragment[elem] +
                                   recieved[len (recieved) -1])
            count -= 1

        return recieved, count

    def send (self, node, data):
        """Send recieved data to main loop.
        """

        if node == DSM_FKBD_MSG:
            graph_data = data.split(' ')
            name = graph_data[0]
            if name in dsm_cur_grphs:
                self.graphs[name].push (graph_data[1], graph_data[2])
        else:
            try:
                wx.PostEvent (self.frame, DsysFifoReadEvent (node, data))
            except:
                pass


class DsysStatusBarMonitor (threading.Thread):
    """This is a class that monitor status bar of GUI.

    This class works as thread. This thread is locked by deafult. If
    there was a writing to status bar, A lock of thread is released.  As
    a result, The sleep that waits clear_interval seconds (3 second by
    default) works. After that, A status bar is cleared, and thread is
    locked.

    """

    def __init__ (self, frame, statusbar):
        """Initalize monitoring by thread.
        """

        threading.Thread.__init__ (self)
        self.frame          = frame
        self.statusbar      = statusbar
        self.Monitoring     = True
        self.clear_interval = 3
        self.last_time      = time.time ()
        self.lock           = threading.Lock ()

        # Lock current thread.
        self.is_lock = True
        self.lock.acquire ()

    def run (self):
        """Start monitoring by thread.
        """

        while (self.Monitoring):
            # clear status bar.
            if (time.time () > self.last_time):
                try:
                    wx.PostEvent (self.frame, DsysStReadEvent (True))
                except:
                    pass

            # Lock thread.
            self.is_lock = True
            self.lock.acquire ()
            # Wait interval time. wait for this time.
            time.sleep (self.clear_interval)

    def stop (self):
        """Stop monitoring by thread.
        """

        self.Monitoring = False
        if self.is_lock == True:
            self.lock.release ()

    def write (self, text):
        """Write specified text to status bar.
        """

        self.statusbar.SetStatusText (text)
        self.last_time = time.time ()

        # run loop for clear status bar.
        if self.is_lock == True:
            self.lock.release ()
            self.is_lock = False

    def clear (self):
        """Clear status bar if any text is written.
        """

        if self.statusbar.GetStatusText () != u'':
            self.statusbar.SetStatusText (u'')


class DsysPanelGraph (wx.Panel):
    """This is class that creates graph that monitor accessing of DISK
    or NETWORK.
    """

    bgcolor = u'black'
    fgcolor = u'yellow'
    grcolor = u'gray'

    is_grid   = False
    is_bar    = True
    is_update = True

    def __init__ (self, parent, title, is_bar):
        """Initialize a graph.
        """

        style = wx.BORDER_SUNKEN
        wx.Panel.__init__ (self, parent, wx.ID_ANY, style=style)
        program_name = os.path.basename (sys.argv[0])

        # style of the graph
        self.is_bar = is_bar

        XLABEL = u'time since %(program_name)s boot [sec]'
        YLABEL = u'data transfer quantity [bytes]'

        # This value determines a size of the graph.
        figure = Figure ((3.0, 3.0), dpi=100)

        self.axes = figure.add_subplot (111)
        self.axes.set_axis_bgcolor (self.bgcolor)
        self.axes.set_title (title, fontsize=7)
        self.axes.set_xlabel (XLABEL % locals (), fontsize=6)
        self.axes.set_ylabel (YLABEL % locals (), fontsize=6)

        pylab.setp (self.axes.get_xticklabels (), fontsize=5)
        pylab.setp (self.axes.get_yticklabels (), fontsize=5)

        grph_width = dsm_redraw_itvl * dsm_grph_indication
        max_width  = int (grph_width / dsm_redraw_itvl)
        width      = dsm_redraw_itvl
        fgcolor    = self.fgcolor
        max_ary    = (max_width -1)

        self.xs = range ((- max_ary * dsm_redraw_itvl), 0, dsm_redraw_itvl)
        self.ys = [0] * max_ary

        # create bar or plot graph
        if (self.is_bar == True):
            self.rects = self.axes.bar (self.xs, self.ys,
                                        width=width,
                                        linewidth=None,
                                        align='edge',
                                        color=fgcolor)
        else:
            self.plots = self.axes.plot (self.xs, self.ys,
                                         linewidth=1,
                                         color=fgcolor)[0]

        self.update_grid ()
        self.update_xybound (self.xs[-1], self.xs[0], 0, 1)

        # Add graph to canvas.
        self.canvas = FigCanvas (self, wx.ID_ANY, figure)
        box = wx.BoxSizer (wx.HORIZONTAL)
        box.Add (self.canvas, 1, wx.EXPAND | wx.ALL)

        self.SetSizer (box)

    def update_grid (self):
        """Redraw grid line
        """

        if self.is_grid == True:
            self.axes.grid (True, color=self.grcolor)

    def update_xybound (self, xmin, xmax, ymin, ymax):
        """Update x and y boundary.
        """

        self.axes.set_xbound (lower=xmin, upper=xmax)
        self.axes.set_ybound (lower=ymin, upper=ymax)

    def clear (self):
        """Clear value of  Y axis in left and right graph.
        """

        xmax = 0
        length = len (self.xs)
        for i in range (length):
            self.ys[i] = 0
            if self.xs[i] > xmax:
                xmax = self.xs[i]
        xmin = self.xs[0]
        ymax = 1
        ymin = 0

        if (self.is_bar == True):
            for x, bar in enumerate (self.rects):
                bar.set_height (self.ys[x])
        else:
            self.plots.set_ydata (np.array (self.ys))

        self.update_xybound (xmin, xmax, ymin, ymax)

        if self.is_update == True:
            self.canvas.draw ()

    def update (self, data):
        """Redraw graph
        """

        xmax = 0
        ymax = 0
        length = len (self.xs)
        for i in range (length):
            if i == (length - 1):
                self.xs[i] = (self.xs[i] + dsm_redraw_itvl)
                self.ys[i] = data
            else:
                self.xs[i] = self.xs[i+1]
                self.ys[i] = self.ys[i+1]
            if self.ys[i] > ymax:
                ymax = self.ys[i]
            if self.xs[i] > xmax:
                xmax = self.xs[i]
        xmin = self.xs[0]
        ymax += 1
        ymin = 0

        if (self.is_bar == True):
            for x, bar in enumerate (self.rects):
                bar.set_x (self.xs[x])
                bar.set_height (self.ys[x])
        else:
            self.plots.set_xdata (np.array (self.xs))
            self.plots.set_ydata (np.array (self.ys))

        self.update_grid ()
        self.update_xybound (xmin, xmax, ymin, ymax)

        # Redraw graph
        if self.is_update == True:
            self.canvas.draw ()


class DsysInterface (wx.Frame):
    """This class seting up GUI interface for dsysmon.

    This GUI interface has following features.

    Menu Buttons:
     - start User OS.
     - display with fullscreen.

    Graphs (FKBD)
     - show access log of disk or network (FKBD).

    Buttons
      - control character string injection timing (FKBD).
      - control checking whether or not hidden file (or size modified
        file) exists (RTKL).
      - clear internal table of RTKL (RTKL).
      - control checking whether or not hidden process exists (LMS).
      - control timing that checks whether or not hidden file exists
        (LMS).

    Textareas
      - show warnings of RTKL (RTKL).
      - show warnings of LMS (LMS).

    """

    LBL_TITLE = u'Dsysmon GUI Interface'

    LBL_FKBD_START = u'Start'
    LBL_FKBD_STOP  = u'Stop'
    LBL_FKBD_INTVL = u'Change'

    LBL_RTKL_START = u'Start'
    LBL_RTKL_STOP  = u'Stop'
    LBL_RTKL_CLR   = u'Clear'

    LBL_LMS_START = u'Start'
    LBL_LMS_STOP  = u'Stop'
    LBL_LMS_INTVL = u'Change'

    LBL_MENU_FILE = u'&File (F)'
    LBL_MENU_VIEW = u'&View (V)'
    LBL_MENU_DSM  = u'&Dsysmon (D)'
    LBL_MENU_QUIT = u'&Quit\tCtrl+Q'
    LBL_MENU_FULL = u'&Fullscreen\tF11'
    LBL_MENU_STRT = u'&Start USER OS\tCtrl+S'
    LBL_MENU_RCLR = u'&Clear RTKL warning'
    LBL_MENU_LCLR = u'&Clear LMS warning'
    LBL_MENU_ACLR = u'&Clear ALL warning'
    LBL_MENU_GCLR = u'&Clear ALL graphs'

    STYLE_BOX  = wx.EXPAND | wx.ALL
    STYLE_BTN  = wx.EXPAND | wx.ALL
    STYLE_TEXT = wx.BORDER_SUNKEN | wx.TE_MULTILINE | wx.TE_RICH2 \
        | wx.TE_READONLY

    SIZE_FRAME = (800, 600)
    SIZE_TEXT  = (600, 30)

    USEROS_PATH_STATUS = u'/sys/kernel/veidt/users/1/state'
    USEROS_PATH_START  = u'/sys/kernel/veidt/users/1/cmd'


    is_fullscreen = False


    def __init__ (self):
        """Initialize GUI interface (constructor).
        """

        title = self.LBL_TITLE
        size  = self.SIZE_FRAME
        wx.Frame.__init__ (self, None, wx.ID_ANY, title, size=size)

        # Setup behavior when close button was pushed
        self.Bind (wx.EVT_CLOSE, self.quit)

        # Setup GUI interface.
        self.setup_menu ()
        self.setup_statusbar ()
        self.setup_panel ()

        self.Centre ()

        # Setup FIFO interface.
        self.Connect (-1, -1, DSM_EVT_FIFO_READ_ID, self.handle_fifo)
        self.fifo = DsysFifo (self)
        self.fifo.start ()

        pattern = "found hidden process, PID (\d+) (\S+) CPU:(\d+)"
        self.lms_msg_pattern = re.compile (pattern)

    def setup_menu (self):
        """Setup menubar of GUI.
        """

        menu = wx.MenuBar ()

        file = wx.Menu ()
        view = wx.Menu ()
        dsym = wx.Menu ()

        quit = wx.MenuItem (file, wx.ID_ANY, self.LBL_MENU_QUIT)
        full = wx.MenuItem (view, wx.ID_ANY, self.LBL_MENU_FULL)
        strt = wx.MenuItem (dsym, wx.ID_ANY, self.LBL_MENU_STRT)
        rclr = wx.MenuItem (dsym, wx.ID_ANY, self.LBL_MENU_RCLR)
        lclr = wx.MenuItem (dsym, wx.ID_ANY, self.LBL_MENU_LCLR)
        aclr = wx.MenuItem (dsym, wx.ID_ANY, self.LBL_MENU_ACLR)
        gclr = wx.MenuItem (dsym, wx.ID_ANY, self.LBL_MENU_GCLR)

        file.AppendItem (quit)
        view.AppendItem (full)
        dsym.AppendItem (strt)
        dsym.AppendSeparator ()
        dsym.AppendItem (rclr)
        dsym.AppendItem (lclr)
        dsym.AppendItem (aclr)
        dsym.AppendSeparator ()
        dsym.AppendItem (gclr)

        menu.Append (file, self.LBL_MENU_FILE)
        menu.Append (view, self.LBL_MENU_VIEW)
        menu.Append (dsym, self.LBL_MENU_DSM)

        self.Bind (wx.EVT_MENU, self.quit, quit)
        self.Bind (wx.EVT_MENU, self.fullscreen, full)
        self.Bind (wx.EVT_MENU, self.useros_start, strt)
        self.Bind (wx.EVT_MENU, self.clear_rtkl, rclr)
        self.Bind (wx.EVT_MENU, self.clear_lms, lclr)
        self.Bind (wx.EVT_MENU, self.clear_all, aclr)
        self.Bind (wx.EVT_MENU, self.clear_grph, gclr)

        # If USER OS has already started, disable useros start button.
        if self.get_useros_status () != 0:
            strt.Enable (False)
        # Disable all textarea clear buttons at once.
        rclr.Enable (False)
        lclr.Enable (False)
        aclr.Enable (False)
        gclr.Enable (False)

        self.menu_start = strt
        self.menu_rclr  = rclr
        self.menu_lclr  = lclr
        self.menu_aclr  = aclr
        self.menu_gclr  = gclr
        self.SetMenuBar (menu)

    def setup_statusbar (self):
        """Setup statusbar of GUI.
        """

        # register a event handler for statusbar.
        self.Connect (-1, -1, DSM_EVT_ST_READ_ID, self.handle_statusbar)

        # create statusbar and start thread for monitoring status of
        # statusbar.
        self.statusbar = DsysStatusBarMonitor (self, self.CreateStatusBar ())
        self.statusbar.start ()

    def get_gtitle (self, name):
        """Get title of the graph from specified argument `name'.

        If specified argument `name' was invalid, return default name
        `DISABLED`.

        """

        default_name = u'DISABLED'
        match        = re.match (DSM_GTITLE_REGEX, name)
        titles       = {'hddlog':'DISK %(numstring)sACCESS',
                        'netlog':'NETWORK %(numstring)sACCESS'}

        if match is None:
            return default_name

        try:
            graph  = match.group (1)
            number = match.group (4)
        except:
            return default_name

        if number == '':
            numstring = ''
        else:
            numstring = '[%(number)s] ' % locals ()

        if graph in titles.keys ():
            return (titles[graph] % locals ())
        else:
            return default_name

    def setup_pfkbd (self, panel):
        """Setup panel for FKBD.
        """

        if "bar" == dsm_grph_style:
            is_bar = True
        else:
            is_bar = False

        LBL_GLEFT = self.get_gtitle (dsm_grph_left)
        LBL_GRGHT = self.get_gtitle (dsm_grph_right)
        self.pnl_left = DsysPanelGraph (panel, LBL_GLEFT, is_bar)
        self.pnl_rght = DsysPanelGraph (panel, LBL_GRGHT, is_bar)

        self.fkbd_grph = wx.BoxSizer (wx.HORIZONTAL)
        self.fkbd_grph.Add (self.pnl_left, 1, self.STYLE_BOX, 5)
        self.fkbd_grph.Add (self.pnl_rght, 1, self.STYLE_BOX, 5)

        # create buttons and textarea for FKBD.
        self.fkbd_bstrt = wx.Button (panel, wx.ID_ANY, self.LBL_FKBD_START)
        self.fkbd_bstop = wx.Button (panel, wx.ID_ANY, self.LBL_FKBD_STOP)
        self.fkbd_bchng = wx.Button (panel, wx.ID_ANY, self.LBL_FKBD_INTVL)
        self.fkbd_titvl = wx.TextCtrl (panel, wx.ID_ANY)

        # disable all button by default.
        for obj in [self.fkbd_bstrt, self.fkbd_bstop, self.fkbd_bchng,
                    self.fkbd_titvl]:
            obj.Enable (False)

        LBL_FKBD_ICTRL = u'control fkbd injection'
        self.fkbd_cbox = wx.StaticBox(panel, wx.ID_ANY, LBL_FKBD_ICTRL)
        self.fkbd_ctrl = wx.StaticBoxSizer(self.fkbd_cbox, orient=wx.HORIZONTAL)

        self.fkbd_ctrl.Add (self.fkbd_bstrt, 1, self.STYLE_BTN, 5)
        self.fkbd_ctrl.Add (self.fkbd_bstop, 1, self.STYLE_BTN, 5)

        LBL_FKBD_INJCT = u'control interval time'
        self.fkbd_ibox = wx.StaticBox (panel, wx.ID_ANY, LBL_FKBD_INJCT)
        self.fkbd_itvl = wx.StaticBoxSizer(self.fkbd_ibox, orient=wx.HORIZONTAL)

        self.fkbd_itvl.Add (self.fkbd_titvl, 1, self.STYLE_BOX, 5)
        self.fkbd_itvl.Add (self.fkbd_bchng, 1, self.STYLE_BTN, 5)

        self.Bind (wx.EVT_BUTTON, self.btn_fkbd_strt, self.fkbd_bstrt)
        self.Bind (wx.EVT_BUTTON, self.btn_fkbd_stop, self.fkbd_bstop)
        self.Bind (wx.EVT_BUTTON, self.btn_fkbd_itvl, self.fkbd_bchng)

        self.fkbd_btn = wx.BoxSizer (wx.HORIZONTAL)
        self.fkbd_btn.Add (self.fkbd_ctrl,  1, self.STYLE_BTN, 5)
        self.fkbd_btn.Add (self.fkbd_itvl,  1, self.STYLE_BTN, 5)

        self.fkbd_frm = wx.StaticBox (panel, wx.ID_ANY, DSM_LBL_FKBD_NOTWORK)
        self.fkbd_box = wx.StaticBoxSizer (self.fkbd_frm, orient=wx.VERTICAL)

        self.fkbd_box.Add (self.fkbd_grph, 1, self.STYLE_BOX, 5)
        self.fkbd_box.Add (self.fkbd_btn,  0, self.STYLE_BTN, 5)

        # Setup font size.
        for obj in [self.fkbd_bstrt, self.fkbd_bstop, self.fkbd_bchng,
                    self.fkbd_frm, self.fkbd_cbox, self.fkbd_ibox]:
            obj.SetFont (wx.Font (8, wx.FONTFAMILY_DEFAULT, wx.FONTSTYLE_NORMAL,
                                  wx.FONTWEIGHT_NORMAL))

    def setup_prtkl (self, panel):
        """Setup panel for RTKL
        """

        id            = wx.ID_ANY
        style         = self.STYLE_TEXT
        size          = self.SIZE_TEXT
        LBL_RTKL_WARN = u'Rootkit Warnings'

        self.rtkl_text = wx.TextCtrl (panel, id, u'', style=style, size=size)
        self.rtkl_tbox = wx.StaticBox (panel, wx.ID_ANY, LBL_RTKL_WARN)
        self.rtkl_tsiz = wx.StaticBoxSizer (self.rtkl_tbox, orient=wx.VERTICAL)

        self.rtkl_tsiz.Add (self.rtkl_text, 1, self.STYLE_BOX, 5)

        self.rtkl_bstrt = wx.Button (panel, wx.ID_ANY, self.LBL_RTKL_START)
        self.rtkl_bstop = wx.Button (panel, wx.ID_ANY, self.LBL_RTKL_STOP)
        self.rtkl_bclr  = wx.Button (panel, wx.ID_ANY, self.LBL_RTKL_CLR)

        # Disable all buttons and textarea.
        for obj in [self.rtkl_text, self.rtkl_bstrt, self.rtkl_bstop,
                    self.rtkl_bclr]:
            obj.Enable (False)

        self.Bind (wx.EVT_BUTTON, self.btn_rtkl_strt, self.rtkl_bstrt)
        self.Bind (wx.EVT_BUTTON, self.btn_rtkl_stop, self.rtkl_bstop)
        self.Bind (wx.EVT_BUTTON, self.btn_rtkl_clr,  self.rtkl_bclr)

        self.rtkl_btn = wx.BoxSizer (wx.VERTICAL)
        self.rtkl_btn.Add (self.rtkl_bstrt, 1, self.STYLE_BTN, 5)
        self.rtkl_btn.Add (self.rtkl_bstop, 1, self.STYLE_BTN, 5)
        self.rtkl_btn.Add (self.rtkl_bclr,  1, self.STYLE_BTN, 5)

        self.rtkl_frm = wx.StaticBox (panel, wx.ID_ANY, DSM_LBL_RTKL_NOTWORK)
        self.rtkl_box = wx.StaticBoxSizer (self.rtkl_frm, orient=wx.HORIZONTAL)
        self.rtkl_box.Add (self.rtkl_tsiz, 1, self.STYLE_BOX, 5)
        self.rtkl_box.Add (self.rtkl_btn,  0, self.STYLE_BOX, 5)

        # Setup font size.
        for obj in [self.rtkl_bstrt, self.rtkl_bstop, self.rtkl_bclr,
                    self.rtkl_frm, self.rtkl_tbox]:
            obj.SetFont (wx.Font (8, wx.FONTFAMILY_DEFAULT, wx.FONTSTYLE_NORMAL,
                                  wx.FONTWEIGHT_NORMAL))
        self.rtkl_text.SetFont (wx.Font (10, wx.FONTFAMILY_DEFAULT,
                                         wx.FONTSTYLE_NORMAL,
                                         wx.FONTWEIGHT_NORMAL))

    def setup_plms (self, panel):
        """Setup panel for LMS
        """

        id           = wx.ID_ANY
        style        = self.STYLE_TEXT
        size         = self.SIZE_TEXT
        LBL_LMS_WARN = u'Rootkit Warnings'

        self.lms_text = wx.TextCtrl (panel, id, u'', style=style, size=size)
        self.lms_tbox = wx.StaticBox (panel, wx.ID_ANY, LBL_LMS_WARN)
        self.lms_tsiz = wx.StaticBoxSizer (self.lms_tbox, orient=wx.VERTICAL)

        self.lms_tsiz.Add (self.lms_text, 1, self.STYLE_BOX, 5)

        self.lms_bstrt = wx.Button (panel, wx.ID_ANY, self.LBL_LMS_START)
        self.lms_bstop = wx.Button (panel, wx.ID_ANY, self.LBL_LMS_STOP)
        self.lms_bitvl = wx.Button (panel, wx.ID_ANY, self.LBL_LMS_INTVL)
        self.lms_titvl = wx.TextCtrl (panel, wx.ID_ANY)

        # Disable all buttons and textarea.
        for obj in [self.lms_text, self.lms_bstrt, self.lms_bstop,
                    self.lms_bitvl, self.lms_titvl]:
            obj.Enable (False)

        self.lms_cbox = wx.BoxSizer (wx.HORIZONTAL)
        self.lms_cbox.Add (self.lms_bstrt, 1, self.STYLE_BTN, 5)
        self.lms_cbox.Add (self.lms_bstop, 1, self.STYLE_BTN, 5)

        LBL_LMS_ICNTRL = u'control lms interval (msec)'
        self.lms_ibox = wx.StaticBox (panel, wx.ID_ANY, LBL_LMS_ICNTRL)
        self.lms_isiz = wx.StaticBoxSizer (self.lms_ibox, orient=wx.HORIZONTAL)

        self.lms_isiz.Add (self.lms_titvl, 0, self.STYLE_BOX, 5)
        self.lms_isiz.Add (self.lms_bitvl, 0, self.STYLE_BTN, 5)

        self.Bind (wx.EVT_BUTTON, self.btn_lms_strt, self.lms_bstrt)
        self.Bind (wx.EVT_BUTTON, self.btn_lms_stop, self.lms_bstop)
        self.Bind (wx.EVT_BUTTON, self.btn_lms_itvl, self.lms_bitvl)

        self.lms_btn = wx.BoxSizer (wx.VERTICAL)
        self.lms_btn.Add (self.lms_cbox, 1, self.STYLE_BTN, 5)
        self.lms_btn.Add (self.lms_isiz, 0, self.STYLE_BTN, 5)

        #self.lms_frm = wx.StaticBox (panel, wx.ID_ANY, DSM_LBL_LMS_NOTWORK)
        #self.lms_box = wx.StaticBoxSizer (self.lms_frm, orient=wx.HORIZONTAL)
        self.lms_box = wx.BoxSizer (wx.HORIZONTAL)
        self.lms_box.Add (self.lms_tsiz, 1, self.STYLE_BOX, 5)
        self.lms_box.Add (self.lms_btn,  0, self.STYLE_BOX, 5)

        # CPUs
        self.lms_cpu_button = []
        self.lms_cpu_label = []
        self.lms_cpu_sizer = []
        self.lms_cpus_label = wx.StaticBox (panel, wx.ID_ANY, "PID of hidden process")
        self.lms_cpus = wx.StaticBoxSizer (self.lms_cpus_label, orient=wx.HORIZONTAL)
        self.cpu_num = dsm_cpu_number
        print "self.cpu_num = " + str (self.cpu_num)
        for i in range (self.cpu_num):
            button = wx.Button (panel, wx.ID_ANY, u'     ')
            self.lms_cpu_button.append (button)
            cpu_id = u'CPU' + str (i)
            label = wx.StaticBox (panel, wx.ID_ANY, cpu_id)
            self.lms_cpu_label.append (label)
            sizer = wx.StaticBoxSizer (label, orient=wx.HORIZONTAL)
            sizer.Add (button, 0, self.STYLE_BOX, 5)
            self.lms_cpu_sizer.append (sizer)
            self.lms_cpus.Add (sizer, 0, self.STYLE_BOX, 5)

        #self.lms_cpu1 = wx.Button (panel, wx.ID_ANY, u'PID-1')
        #self.lms_cpu2 = wx.Button (panel, wx.ID_ANY, u'PID-2')
        #self.lms_cpu0.SetBackgroundColour ("#ff0000")
        #self.lms_cpu1.SetBackgroundColour ("#ff0000")

        #self.lms_cpu1label = wx.StaticBox (panel, wx.ID_ANY, u'CPU1')
        #self.lms_cpu1siz = wx.StaticBoxSizer (self.lms_cpu1label, orient=wx.HORIZONTAL)
        #self.lms_cpu1siz.Add (self.lms_cpu1, 0, self.STYLE_BOX, 5)

        #self.lms_cpus = wx.BoxSizer (wx.HORIZONTAL)
        #self.lms_cpus.Add (self.lms_cpu0, 0, self.STYLE_BOX, 5)
        #self.lms_cpus.Add (self.lms_cpu1siz, 0, self.STYLE_BOX, 5)
        #self.lms_cpus.Add (self.lms_cpu2, 0, self.STYLE_BOX, 5)

        self.lms_frm = wx.StaticBox (panel, wx.ID_ANY, DSM_LBL_LMS_NOTWORK)
        self.lms_box2 = wx.StaticBoxSizer (self.lms_frm, orient=wx.VERTICAL)
        self.lms_box2.Add (self.lms_box,  0, self.STYLE_BOX, 5)
        self.lms_box2.Add (self.lms_cpus, 0, self.STYLE_BOX, 5)

        # Setup font size.
        for obj in [self.lms_bstrt, self.lms_bstop, self.lms_bitvl,
                    self.lms_frm, self.lms_tbox, self.lms_ibox, self.lms_cpus_label] + \
                    self.lms_cpu_label :
            obj.SetFont (wx.Font (8, wx.FONTFAMILY_DEFAULT, wx.FONTSTYLE_NORMAL,
                                  wx.FONTWEIGHT_NORMAL))
        self.lms_text.SetFont (wx.Font (10, wx.FONTFAMILY_DEFAULT,
                                        wx.FONTSTYLE_NORMAL,
                                        wx.FONTWEIGHT_NORMAL))
        for obj in self.lms_cpu_button :
            obj.SetFont (wx.Font (14, wx.FONTFAMILY_DEFAULT, wx.FONTSTYLE_NORMAL,
                                  wx.FONTWEIGHT_NORMAL))


    def set_lms_cpu (self, data) :
        result = self.lms_msg_pattern.search (data)
        if result == None :
            return
        pid = int (result.group (1))
        cpu = int (result.group (3))
        #print "pid = " + str (pid)
        #print "cpu = " + str (cpu)
        for i in range (self.cpu_num) :
            self.lms_cpu_button[i].SetBackgroundColour ("white")
            self.lms_cpu_button[i].SetLabel (u'     ')
        if cpu < self.cpu_num :
            self.lms_cpu_button[cpu].SetBackgroundColour ("#ff0000")
            self.lms_cpu_button[cpu].SetForegroundColour ("white")
            self.lms_cpu_button[cpu].SetLabel (str (pid))

    def setup_panel (self):
        """Setup GUI all panels (FKBD, RTKL and LMS)
        """

        panel = wx.Panel (self, wx.ID_ANY)
        self.vbox  = wx.BoxSizer (wx.VERTICAL)

        self.setup_pfkbd (panel)
        self.setup_prtkl (panel)
        self.setup_plms (panel)

        self.vbox.Add (self.fkbd_box, 0, self.STYLE_BOX, 5)
        self.vbox.Add (self.rtkl_box, 1, self.STYLE_BOX, 5)
        #self.vbox.Add (self.lms_box , 1, self.STYLE_BOX, 5)
        self.vbox.Add (self.lms_box2 , 1, self.STYLE_BOX, 5)

        self.vbox.Fit (self)
        panel.SetSizer (self.vbox)

    def quit (self, event):
        """Quit this program
        """

        # stop any running threads.
        self.fifo.stop ()
        self.statusbar.stop ()
        # destory own frame.
        self.Destroy ()

    def fullscreen (self, event):
        """Toggle fullscreen
        """

        STYLE = wx.FULLSCREEN_NOBORDER | wx.FULLSCREEN_NOCAPTION \
            | wx.FULLSCREEN_NOTOOLBAR

        if self.is_fullscreen == True:
            self.ShowFullScreen (False)
            self.is_fullscreen = False
        else:
            self.ShowFullScreen (True, style=STYLE)
            self.is_fullscreen = True

    def get_useros_status (self):
        """Return status of USER os (running or not)

         - running     :  1
         - not running :  0
         - error       : -1

        """

        if not os.path.exists (self.USEROS_PATH_STATUS):
            return -1

        # get user os status
        try:
            f = open (self.USEROS_PATH_STATUS)
            status = f.read ().rstrip ('\n')
        finally:
            f.close ()

        if status.isdigit () == True:
            return int (status)
        else:
            return -1

    def useros_start (self, event):
        """Start USER OS.
        """

        MSG            = u'start USER OS'
        USEROS_RESTART = u'restart'

        # start user os.
        status = self.get_useros_status ()
        if status == 0:
            self.statusbar.write (MSG)
            try:
                f = open (self.USEROS_PATH_START, 'w')
                f.write (USEROS_RESTART)
            finally:
                f.close ()
                self.menu_start.Enable (False)

    def clear_rtkl (self, event):
        """Clear textctl of RTKL.
        """

        self.rtkl_text.Clear ()

    def clear_lms (self, event):
        """Clear textctl of LMS.
        """

        self.lms_text.Clear ()

    def clear_all (self, event):
        """Clear textctl of RTKL and LMS.
        """

        self.rtkl_text.Clear ()
        self.lms_text.Clear ()

    def clear_grph (self, event):
        """Clear all graph Y value of FKBD.
        """

        for g in [self.pnl_left, self.pnl_rght]:
            g.clear ()

    def btn_fkbd_strt (self, event):
        """Start FKBD injection
        """

        MSG      = u'fkbd injection start'
        MSG_STRT = u'start'

        self.statusbar.write (MSG)
        self.fifo.write (DSM_FKBD_CTL, MSG_STRT)

        # Update button status.
        self.fkbd_bstrt.Enable (False)
        self.fkbd_bstop.Enable (True)

    def btn_fkbd_stop (self, event):
        """Stop FKBD injection
        """

        MSG      = u'fkbd injection stop'
        MSG_STOP = u'stop'

        self.statusbar.write (MSG)
        self.fifo.write (DSM_FKBD_CTL, MSG_STOP)

        # Update button status.
        self.fkbd_bstrt.Enable (True)
        self.fkbd_bstop.Enable (False)

    def btn_fkbd_itvl (self, event):
        """Change injection interval of FKBD.
        """

        interval = self.fkbd_titvl.GetValue ()

        MSG_OK   = u'fkbd injection interval = %(interval)s.'
        MSG_NG   = u'specified text %(interval)s is not integer.'
        MSG_ITVL = u'interval %(interval)s'

        if interval.isdigit () == True:
            self.statusbar.write (MSG_OK % locals ())
            self.fifo.write (DSM_FKBD_CTL, MSG_ITVL % locals ())
        else:
            self.statusbar.write (MSG_NG % locals ())

    def btn_rtkl_strt (self, event):
        """Start checking of rootkit by RTKL.
        """

        MSG = u'rtkl check start'
        MSG_STRT = u'start'

        self.statusbar.write (MSG)
        self.fifo.write (DSM_RTKL_CTL, MSG_STRT)

        # Update button status.
        self.rtkl_bstrt.Enable (False)
        self.rtkl_bstop.Enable (True)

        # Update RTKL status.
        self.rtkl_frm.SetLabel (DSM_LBL_RTKL_WORK)


    def btn_rtkl_stop (self, event):
        """Stop checking of rootkit by RTKL.
        """

        MSG = u'rtkl check stop'
        MSG_STOP = u'stop'

        self.statusbar.write (MSG)
        self.fifo.write (DSM_RTKL_CTL, MSG_STOP)

        # Update button status
        self.rtkl_bstrt.Enable (True)
        self.rtkl_bstop.Enable (False)

        # Update RTKL status.
        self.rtkl_frm.SetLabel (DSM_LBL_RTKL_STOP)

    def btn_rtkl_clr (self, event):
        """Clear RTKL internal table.
        """

        MSG = u'rtkl internal table clear'
        MSG_CLR = u'clear'

        self.statusbar.write (MSG)
        self.fifo.write (DSM_RTKL_CTL, MSG_CLR)

    def btn_lms_strt (self, event):
        """Start checking of rootkit by LMS.
        """

        MSG = u'lms check start'
        MSG_STRT = u'start'

        self.statusbar.write (MSG)
        self.fifo.write (DSM_LMS_CTL, MSG_STRT)

        # Update button status.
        self.lms_bstrt.Enable (False)
        self.lms_bstop.Enable (True)

        # Update LMS status.
        self.lms_frm.SetLabel (DSM_LBL_LMS_WORK)

    def btn_lms_stop (self, event):
        """Stop checking of rootkit by LMS.
        """

        MSG = u'lms check stop'
        MSG_STOP = u'stop'

        self.statusbar.write (MSG)
        self.fifo.write (DSM_LMS_CTL, MSG_STOP)

        # Update button status.
        self.lms_bstrt.Enable (True)
        self.lms_bstop.Enable (False)

        # Update LMS status.
        self.lms_frm.SetLabel (DSM_LBL_LMS_STOP)

    def btn_lms_itvl (self, event):
        """Change checking interval of LMS.
        """

        interval     = self.lms_titvl.GetValue ()
        MIN_INTERVAL = 100

        MSG_OK = u'lms interval = %(interval)s'
        MSG_NG_INT = u'specified text \'%(interval)s\' is not integer.'
        MSG_NG_MIN = (u'specified value is too small (%(interval)s). ' +
                      'specify greater than %(MIN_INTERVAL)d')
        MSG_ITVL = u'interval %(interval)s'

        if interval.isdigit () == True:
            if int (interval) < MIN_INTERVAL:
                self.statusbar.write (MSG_NG_MIN % locals ())
            else:
                self.statusbar.write (MSG_OK % locals ())
                self.fifo.write (DSM_LMS_CTL, MSG_ITVL % locals ())
        else:
            self.statusbar.write (MSG_NG_INT % locals ())

    def handle_statusbar (self, msg):
        """Handle events from statusbar.
        """

        # clear statusbar.
        is_clear =  msg.clear
        if is_clear == True:
            self.statusbar.clear ()

    def handle_fifo (self, msg):
        """Handle events from fifo.
        """

        type = msg.type
        data = msg.data

        if type in [DSM_RTKL_MSG, DSM_LMS_MSG]:
            # write warning from RTKL or LMS.
            self.warn (type, data)
        elif type in dsm_cur_grphs:
            # update graphs.
            self.update_grph (type, data)
        else:
            # unknown type.
            pass

    def warn (self, type, data):
        """Write warings from RTKL or LMS.
        """

        target = None
        if type == DSM_RTKL_MSG:
            target = self.rtkl_text
        elif type == DSM_LMS_MSG:
            target = self.lms_text
            self.set_lms_cpu (data) 

        if target != None:
            target.SetEditable (True)
            if target.IsEmpty ():
                target.AppendText (data)
            else:
                target.AppendText ('\n' + data)
            target.SetEditable (False)

    def update_grph (self, type, data):
        """Update graph.
        """

        MSG_NG = "unknown type (%(type)s)."

        if type == dsm_grph_left:
            self.pnl_left.update (data)
        elif type == dsm_grph_right:
            self.pnl_rght.update (data)
        else:
            print (MSG_NG % locals ())


def dsm_usage_show (output):
    """Show help message for this program.
    """

    program_name = os.path.basename (sys.argv[0])
    print >> output, (DSM_MSG_USAGE % locals ())

def dsm_version_show (output):
    """Show program version
    """

    version = DSM_VERSION_STRING
    timestamp = str (time.ctime(os.path.getmtime (__file__)))
    print >> output, (DSM_MSG_VERRION % locals ())

def dsm_option_parse ():
    """Parse command line options

    This program interrupts following options currently.

     -c ... specify the number of cpu cores.
     -d ... specify directory that stores fifo device name.
     -g ... specify graph style. it accepts \"bar\" or \"plot\".
            default value is \"plot\".
     -h ... show help messages.
     -i ... specify redraw interval of the graph.
     -l ... specify name of the left graph.
     -r ... specify name of the right graph.
     -s ... specify scale of the graph.
     -v ... show program version.
    """

    # Parse options.
    # If recieved unknown option, show usage and exit.
    try:
        sopt = "c:d:g:hi:l:r:s:v"
        lopt = ["help", "version", "device", "graph", "interval", "left",
                "right", "scale"]
        (opts, args) = getopt.getopt (sys.argv[1:], sopt, longopts=lopt)
    except getopt.GetoptError:
        dsm_usage_show (sys.stderr)
        sys.exit (1)

    # Interrupt options.
    for opt, arg in opts:
        # Show help message and exit.
        if opt in ("-h", "--help"):
            dsm_usage_show (sys.stdout)
            sys.exit (0);

        # Show version string and exit.
        if opt in ("-v", "--version"):
            dsm_version_show (sys.stdout)
            sys.exit (0);

        # Set graph style (bar or plot)
        if opt in ("-g", "--graph"):
            global dsm_grph_style
            if arg in ["bar", "plot"]:
                dsm_grph_style = arg.rstrip ('\n')
            else:
                print >> sys.stderr, (DSM_ERR_NOGRAPH % locals ())
                sys.exit (1)

        # Set redraw interval.
        if opt in ("-i", "--interval"):
            global dsm_redraw_itvl
            if arg.isdigit () == True:
                dsm_redraw_itvl = int (arg)
            else:
                print >> sys.stderr, (DSM_ERR_NOTDIGIT % locals ())
                sys.exit (1)

        # Set the number of cpu cores.
        if opt in ("-c", "--cpu"):
            global dsm_cpu_number
            if arg.isdigit () == True:
                dsm_cpu_number = int (arg)
                print "dsm_cpu_number = " + str (dsm_cpu_number)
            else:
                print >> sys.stderr, (DSM_ERR_NOTDIGIT % locals ())
                sys.exit (1)

        # Set name of the left graph.
        if opt in ("-l", "--left"):
            global dsm_grph_left
            if re.match (DSM_GTITLE_REGEX, arg) != None:
                dsm_grph_left = arg
            else:
                print >> sys.stderr, (DSM_ERR_INVALID_GTITLE % locals ())
                sys.exit (1)

        # Set name of the right graph.
        if opt in ("-r", "--right"):
            global dsm_grph_right
            if re.match (DSM_GTITLE_REGEX, arg) != None:
                dsm_grph_right = arg
            else:
                print >> sys.stderr, (DSM_ERR_INVALID_GTITLE % locals ())
                sys.exit (1)

        # Set indication of the graph
        if opt in ("-s", "--scale"):
            global dsm_grph_indication
            if arg.isdigit () == True:
                dsm_grph_indication = int (arg)
            else:
                print >> sys.stderr, (DSM_ERR_NOTDIGIT % locals ())
                sys.exit (1)

        # Get directory that stores fifos.
        if opt in ("-d", "--device"):
            path = arg
            # Check whether or not path exists.
            if os.path.isdir (path):
                dsm_fifo_paths["base"] = path
            else:
                print >> sys.stderr, (DSM_ERR_NOFIFO % locals ())
                sys.exit (1)

def dsm_main ():
    """Check command line option and run gui interface.
    """

    # Parse and check commandl line options.
    dsm_option_parse ()

    # setup graph name.
    if dsm_grph_left == dsm_grph_right:
        print >> sys.stderr, (DSM_ERR_GTITLE_SAMENAME)
        sys.exit (1)

    dsm_cur_grphs.append (dsm_grph_left)
    dsm_cur_grphs.append (dsm_grph_right)

    # If fifo base directory was not registered,
    if not ("base" in dsm_fifo_paths):
        print >> sys.stderr, (DSM_ERR_NODEV % locals ())
        sys.exit (1)

    # Run GUI interface
    application = wx.PySimpleApp ()
    application.frame = DsysInterface ()
    application.frame.Show ()
    application.MainLoop ()


if __name__ == '__main__':
    # Start here.
    dsm_main ()

# dsym-gui.py ends here
