/*
 * Copyright (c) 2013 JST DEOS R&D Center
 */
/*
 * rpld_reciever.c:
 *
 * Recieve tty logs that rpld sent.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdint.h>
#include <stdbool.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <unistd.h>
#include <fcntl.h>
#include <getopt.h>
#include <libgen.h>
#include <errno.h>
#include <limits.h>

/* Name of this program.  */
char* rr_program_name = NULL;

/* Version of this program. */
const char* rr_program_version = "0.0.1";

/* Output debug purpose message if true. */
bool rr_is_debug = false;

/* Filename that write recived tty logs. */
char* rr_log_filename = NULL;


#define DEBUG(args...) \
	do { if (rr_is_debug) fprintf(stdout, args); } while(0)


static void
rr_version_show (FILE* stream)
{
	fprintf (stream, "Version %s (%s %s)\n",
		 rr_program_version, __DATE__, __TIME__);
}

static void
rr_usage_show (FILE* stream)
{
	fprintf (stream,
		 "Usage: %s\n"
		 "Options:\n"
		 "	-o <filename>\n"
		 "		Specify output file name of the recieved\n"
		 "		tty logging.\n"
		 "	-p <port-number>\n"
		 "		Specify the port number o this TCP server.\n"
		 "		The default value is 10080."
		 "	-d\n"
		 "		Show debug purpose message too.\n"
		 "	-h\n"
		 "		Show this message.\n"
		 "	-v\n"
		 "		Show version string.\n",
		 rr_program_name);
}

static int
rr_string_to_int (char* str, int base)
{
	int   num;
	char* ep;
	long  val;

	errno = 0;
	val = strtol (str, &ep, base);
	if ((errno == ERANGE && (val == LONG_MAX || val == LONG_MIN)) ||
	    (errno != 0 && val == 0)) {
		perror ("strtol");
		return -1;
	}

	if (ep == optarg) {
		fprintf (stderr, "No digits were found.\n");
		return -1;
	}

	return (num = val);
}

/* Write recived tty logs to local file. */
static void
rr_log_write (char* buffer, int len)
{
	FILE*		fp    = NULL;
	const size_t	count = 1;

	if ((fp = fopen (rr_log_filename, "a")) == NULL) {
		perror ("fopen");
		return ;
	}


	if (fwrite (buffer, len, count, fp) != count) {
		perror ("fwrite");
	}

	fclose (fp);
}

static bool
rr_server_init (int port_number)
{
	bool			status = true;
	int			err;
	int			size = 0;
	int			connect_fd = -1, listen_fd = -1;
	char			port_str[10];
	socklen_t		len;
	struct addrinfo		hints, *res;
	struct sockaddr_in	client;

	char buffer[1024];

	memset (&hints, 0, sizeof (hints));
	hints.ai_family = AF_INET;
	hints.ai_flags	= AI_PASSIVE;
	hints.ai_socktype = SOCK_STREAM;

	if (snprintf (port_str, sizeof (port_str), "%d", port_number) < 0) {
		perror ("snprintf");
		return false;
	}

	if ((err = getaddrinfo (NULL, port_str, &hints, &res)) != 0) {
		fprintf (stderr, "getaddrinfo : '%s'\n", gai_strerror(err));
		return false;
	}

	if ((listen_fd = socket (res->ai_family, res->ai_socktype, 0)) < 0) {
		perror ("socket");
		return false;
	}

	if (bind (listen_fd, res->ai_addr, res->ai_addrlen) < 0) {
		perror ("bind");
		status = false;
		goto exit;
	}
	freeaddrinfo (res);

	if (listen (listen_fd, 5) < 0) {
		perror ("listen");
		status = false;
		goto exit;
	}

	len = sizeof (client);
	while ((connect_fd = accept (listen_fd, (struct sockaddr *)&client,
				     &len)) >= 0) {
		do {
			memset (buffer, 0, sizeof (buffer));
			size = read (connect_fd, buffer, sizeof (buffer));
			if (size < 0) {
				fprintf(stderr, "read is failed.\n");
				break;
			}
			if (rr_log_filename != NULL) {
				rr_log_write (buffer, size);
			}
		} while (size > 0);
	}

	if (connect_fd < 0) {
		perror ("accept");
		status = false;
		goto exit;
	}

exit:
	if (connect_fd > 0 && close (connect_fd) < 0) {
		perror ("close");
	}

	if (listen_fd > 0 && close (listen_fd) < 0) {
		perror ("close");
	}

	return status;
}

int
main (int argc, char* argv[])
{
	static struct option opts[] = {
		{"--debug",		no_argument,		NULL,	'd'},
		{"--help",		no_argument,		NULL,	'h'},
		{"--output-file",	required_argument,	NULL,	'o'},
		{"--port",		required_argument,	NULL,	'p'},
		{"--version",		no_argument,		NULL,	'v'},
		{ NULL,			0,			NULL,	0},
	};

	int c;
	int port_number = 10080;
	while ((c = getopt_long (argc, argv, "dho:p:v", opts, NULL)) != EOF) {
		switch (c) {
		case 'd':
			rr_is_debug = true;
			break;
		case 'h':
			rr_usage_show (stdout);
			exit (EXIT_SUCCESS);
		case 'o':
			rr_log_filename = optarg;
			break;
		case 'p':
			port_number = rr_string_to_int (optarg, 10);
			break;
		case 'v':
			rr_version_show (stdout);
			exit (EXIT_SUCCESS);
		default:
			rr_usage_show (stderr);
			exit (EXIT_FAILURE);
		}
	}

	DEBUG ("D: port number is %d.\n", port_number);
	if (rr_log_filename == NULL) {
		DEBUG ("D: do not save tty log to local file.\n");
	} else {
		DEBUG ("D: save log file to %s.\n", rr_log_filename);
	}

	if (rr_server_init (port_number) == false) {
		return EXIT_FAILURE;
	}

	return EXIT_SUCCESS;
}

/* rpld_reciever.c ends here. */
